@@ -41,12 +41,81 @@ accounts <- function(server = NULL) {
   data.frame(name = names, server = servers, stringsAsFactors = FALSE)
 }
 
+#' Connect Api User Account
+#'
+#' Connect a user account to the package using an API key for authentication
+#' so that it can be used to deploy and manage
+#' applications on behalf of the account.
+#'
+#' @param account A name for the account to connect. Optional.
+#' @param server The server to connect to. Optional if there is only one server
+#'   registered.
+#' @param apiKey The API key used to authenticate the user
+#' @param quiet Whether or not to show messages and prompts while connecting the
+#'   account.
+#'
+#' @details This function configures the user to connect using an apiKey in
+#' the http auth headers instead of a token. This is less secure but may
+#' be necessary when the client is behind a proxy or otherwise unable to
+#' authenticate using a token.
+#'
+#' @family Account functions
+#' @export
+connectApiUser <- function(account = NULL, server = NULL, apiKey = NULL, quiet = FALSE) {
+  # if server isn't specified, look up the default
+  if (is.null(server)) {
+    target <- getDefaultServer(local = TRUE)
+  } else {
+    target <- serverInfo(server)
+  }
+
+  if (is.null(target)) {
+    stop("You must specify a server to connect to.")
+  }
+
+  # if account is specified and we already know about the account, get the User
+  # ID so we can prefill authentication fields
+  userId <- 0
+  userAccounts <- accounts(target$name)
+  if (!is.null(account) && !is.null(userAccounts)) {
+    if (account %in% userAccounts[,"name"]) {
+      accountDetails <- accountInfo(account, target$name)
+      userId <- accountDetails$accountId
+      if (!quiet) {
+        message("The account '",  account, "' is already registered; ",
+                "attempting to reconnect it.")
+      }
+    }
+  }
+
+  user <- getAuthedUser(serverUrl = target$url,
+                        apiKey = apiKey)
+
+  if (is.null(user)) {
+    stop("Unable to fetch user data for provided API key")
+  }
+
+  # write the user info
+  registerUserApiKey(serverName = target$name,
+                    accountName = account,
+                    userId = user$id,
+                    apiKey = apiKey)
+
+  if (!quiet) {
+    message("\nAccount registered successfully: ", account)
+  }
+}
+
 #' Connect User Account
 #'
 #' Connect a user account to the package so that it can be used to deploy and
 #' manage applications on behalf of the account.
 #'
 #' @param account A name for the account to connect. Optional.
+#' @param launch.browser If true, the system's default web browser will be
+#'   launched automatically after the app is started. Defaults to `TRUE` in
+#'   interactive sessions only. If a function is passed, it will be called
+#'   after the app is started, with the app URL as a paramter.
 #' @param server The server to connect to. Optional if there is only one server
 #'   registered.
 #' @param quiet Whether or not to show messages and prompts while connecting the
@@ -61,7 +130,8 @@ accounts <- function(server = NULL) {
 #'
 #' @family Account functions
 #' @export
-connectUser <- function(account = NULL, server = NULL, quiet = FALSE) {
+connectUser <- function(account = NULL, server = NULL, quiet = FALSE,
+                        launch.browser = getOption("rsconnect.launch.browser", interactive())) {
   # if server isn't specified, look up the default
   if (is.null(server)) {
     target <- getDefaultServer(local = TRUE)
@@ -95,13 +165,19 @@ connectUser <- function(account = NULL, server = NULL, quiet = FALSE) {
             "manually by visiting ", token$claim_url, ".")
     message("Waiting for authentication...")
   }
-  utils::browseURL(token$claim_url)
+
+  if (isTRUE(launch.browser))
+    utils::browseURL(token$claim_url)
+  else if (is.function(launch.browser))
+    launch.browser(token$claim_url)
 
   # keep trying to authenticate until we're successful
   repeat {
     Sys.sleep(1)
-    user <- getUserFromRawToken(target$url, token$token, token$private_key,
-                                target$certificate)
+    user <- getAuthedUser(serverUrl = target$url,
+                          token = token$token,
+                          privateKey = token$private_key,
+                          serverCertificate = target$certificate)
     if (!is.null(user))
       break
   }
@@ -274,19 +350,28 @@ getAuthToken <- function(server, userId = 0) {
     claim_url = response$token_claim_url)
 }
 
-# given a server URL and raw information about an auth token, return the user
-# who owns the token, if it's claimed, and NULL if the token is unclaimed.
+# given a server URL and auth parameters, return the user
+# who owns the auth if it's valid/claimed, and NULL if invalid/unclaimed.
 # raises an error on any other HTTP error.
 #
 # this function is used by the RStudio IDE as part of the workflow which
 # attaches a new Connect account.
-getUserFromRawToken <- function(serverUrl, token, privateKey,
-                                serverCertificate = NULL) {
-  # form a temporary client from the raw token
-  connect <- connectClient(service = serverUrl, authInfo =
-                           list(token = token,
-                                private_key = as.character(privateKey),
-                                certificate = serverCertificate))
+getAuthedUser <- function(serverUrl,
+                          token = NULL,
+                          privateKey = NULL,
+                          serverCertificate = NULL,
+                          apiKey = NULL) {
+  authInfo <- NULL
+  if (!is.null(apiKey)) {
+    authInfo <- list(apiKey = apiKey)
+  } else {
+    authInfo <- list(token = token,
+                     private_key = as.character(privateKey),
+                     certificate = serverCertificate)
+  }
+
+  # form a temporary client from the authInfo
+  connect <- connectClient(service = ensureConnectServerUrl(serverUrl), authInfo)
 
   # attempt to fetch the user
   user <- NULL
@@ -302,6 +387,32 @@ getUserFromRawToken <- function(serverUrl, token, privateKey,
   user
 }
 
+# passthrough function for compatibility with old IDE versions
+getUserFromRawToken <- function(serverUrl,
+                                token,
+                                privateKey,
+                                serverCertificate = NULL) {
+  getAuthedUser(serverUrl = serverUrl,
+                token = token,
+                privateKey = privateKey,
+                serverCertificate = serverCertificate)
+}
+
+registerUserApiKey <- function(serverName, accountName, userId, apiKey) {
+  # write the user info
+  configFile <- accountConfigFile(accountName, serverName)
+  dir.create(dirname(configFile), recursive = TRUE, showWarnings = FALSE)
+  write.dcf(list(username = accountName,
+                 accountId = userId,
+                 apiKey = apiKey,
+                 server = serverName),
+            configFile)
+
+  # set restrictive permissions on it if possible
+  if (identical(.Platform$OS.type, "unix"))
+    Sys.chmod(configFile, mode="0600")
+}
+
 registerUserToken <- function(serverName, accountName, userId, token,
                               privateKey) {
   # write the user info
@@ -365,8 +476,17 @@ resolveAccount <- function(account, server = NULL) {
   }
 }
 
-isShinyapps <- function(accountInfo) {
-  identical(accountInfo$server, "shinyapps.io")
+isShinyapps <- function(server) {
+  identical(server, "shinyapps.io")
+}
+
+isRPubs <- function(server) {
+  identical(server, "rpubs.com")
+}
+
+isConnectInfo <- function(accountInfo = NULL, server = NULL) {
+  host <- if (is.null(accountInfo)) server else accountInfo$server
+  !isShinyapps(host) && !isRPubs(host)
 }
 
 stopWithNoAccount <- function() {
