@@ -9,12 +9,16 @@
 #' @return
 #' Returns a data frame with the following columns:
 #' \tabular{ll}{
-#' `name` \tab Name of application \cr
-#' `url` \tab URL where application can be accessed\cr
-#' `config_url` \tab URL where application can be configured\cr
-#' `status` \tab Current status of application. Valid values are
-#' `pending`, `deploying`, `running`, `terminating`, and
-#' `terminated`.
+#' `id`         \tab Application unique id \cr
+#' `name`       \tab Name of application \cr
+#' `url`        \tab URL where application can be accessed \cr
+#'
+#' `status`     \tab Current status of application. Valid values are `pending`,
+#'                   `deploying`, `running`, `terminating`, and `terminated` \cr
+#' `size`       \tab Instance size (small, medium, large, etc.) (on
+#'                   ShinyApps.io) \cr
+#' `instances`  \tab Number of instances (on ShinyApps.io) \cr
+#' `config_url` \tab URL where application can be configured \cr
 #' }
 #' @note To register an account you call the [setAccountInfo()] function.
 #' @examples
@@ -39,41 +43,83 @@ applications <- function(account = NULL, server = NULL) {
   serverDetails <- serverInfo(accountDetails$server)
   client <- clientForAccount(accountDetails)
 
-  # retreive applications
+  isConnect <- isConnectInfo(accountInfo = accountDetails)
+
+  # retrieve applications
   apps <- client$listApplications(accountDetails$accountId)
 
+  keep <- if (isConnect) {
+    c(
+      'id',
+      'name',
+      'url',
+      'build_status',
+      'created_time',
+      'last_deployed_time',
+      'guid'
+    )
+  } else {
+    c(
+      'id',
+      'name',
+      'url',
+      'status',
+      'created_time',
+      'updated_time',
+      'deployment'
+    )
+  }
   # extract the subset of fields we're interested in
-  res <- lapply(apps, `[`, c('id', 'name', 'url', 'status', 'created_time',
-                             'updated_time', 'deployment'))
+  res <- lapply(apps, `[`, keep)
 
-  # promote the size and instance data to first-level fields
-  res <- lapply(res, function(x) {
-    if (! is.null(x$deployment)) {
+  res <- if (isConnect) {
+    lapply(res, function(x){
+      # set size and instance to NA since Connect doesn't return this info
+      x$size <- NA
+      x$instances <- NA
+      x
+    })
+  } else {
+    lapply(res, function(x) {
+      # promote the size and instance data to first-level fields
       x$size <- x$deployment$properties$application.instances.template
+      if (is.null(x$size))
+        x$size <- NA
       x$instances <- x$deployment$properties$application.instances.count
       if (is.null(x$instances))
         x$instances <- NA
+      x$deployment <- NULL
+      x$guid <- NA
+      x
+    })
+  }
+
+  # The config URL may be provided by the server at some point, but for now
+  # infer it from the account type
+  res <- lapply(res, function(row) {
+    if (isConnect) {
+      prefix <- sub("/__api__", "", serverDetails$url)
+      row$config_url <- paste(prefix, "connect/#/apps", row$id, sep = "/")
     } else {
-      x$size <- NA
-      x$instances <- NA
+      row$config_url <- paste("https://www.shinyapps.io/admin/#/application", row$id, sep = "/")
     }
-    x$deployment <- NULL
-
-    # this may be provided by the server at some point, but for now infer it
-    # from the account type
-    x$config_url <- if (isShinyapps(accountDetails))
-      paste("https://www.shinyapps.io/admin/#/application", x$id, sep = "/")
-    else
-      sub("/__api__", paste("/connect/#/apps", x$id, sep = "/"),
-          serverDetails$url)
-
-    return(x)
+    row
   })
 
   # convert to data frame
-  res <- do.call(rbind, res)
+  rbindWithoutFactors <- function(...){
+    rbind.data.frame(..., stringsAsFactors = FALSE)
+  }
+  res <- do.call(rbindWithoutFactors, res)
 
-  as.data.frame(res, stringsAsFactors = FALSE)
+  # Ensure the Connect and ShinyApps.io data frames have same column names
+  idx <- match("last_deployed_time", names(res))
+  if(!is.na(idx)) names(res)[idx] <- "updated_time"
+
+  idx <- match("build_status", names(res))
+  if (!is.na(idx)) names(res)[idx] <- "status"
+
+  return(res)
 }
 
 resolveApplication <- function(accountDetails, appName) {
@@ -147,6 +193,8 @@ streamApplicationLogs <- function(authInfo, applicationId, entries, skip) {
 #'   if only one application deployment was made from `appPath`.
 #' @param account The account under which the application was deployed. May be
 #'   omitted if only one account is registered on the system.
+#' @param server Server name. Required only if you use the same account name on
+#'   multiple servers.
 #' @param entries The number of log entries to show. Defaults to 50 entries.
 #' @param streaming Whether to stream the logs. If `TRUE`, then the
 #'   function does not return; instead, log entries are written to the console
@@ -157,11 +205,11 @@ streamApplicationLogs <- function(authInfo, applicationId, entries, skip) {
 #'
 #' @export
 showLogs <- function(appPath = getwd(), appFile = NULL, appName = NULL,
-                     account = NULL, entries = 50, streaming = FALSE) {
+                     account = NULL, server = NULL, entries = 50, streaming = FALSE) {
 
   # determine the log target and target account info
-  target <- deploymentTarget(appPath, appName, NULL, NULL, account)
-  accountDetails <- accountInfo(target$account)
+  target <- deploymentTarget(appPath, appName, NULL, NULL, account, server)
+  accountDetails <- accountInfo(target$account, target$server)
   client <- lucidClient(shinyappsServerInfo()$url, accountDetails)
   application <- getAppByName(client, accountDetails, target$appName)
   if (is.null(application))
@@ -191,7 +239,84 @@ showLogs <- function(appPath = getwd(), appFile = NULL, appName = NULL,
     }
   } else {
     # if not streaming, poll for the entries directly
-    logs <- client$getLogs(application$id, entries, FALSE)
+    logs <- client$getLogs(application$id, entries)
     cat(logs)
   }
 }
+
+#' Sync Application Metadata
+#'
+#' Update the metadata for requested application across all deployments
+#'
+#' @param appPath The path to the directory or file that was deployed.
+#'
+#' @note This function does not update metadata for Shiny and rpubs apps
+#'
+#' @export
+syncAppMetadata <- function(appPath) {
+  if (is.null(appPath) || !file.exists(appPath)) {
+    stop("appPath is null or does not exist")
+  }
+
+  # get all of the currently known deployments
+  deploys <- deployments(appPath)
+
+  syncWindow = getOption("rsconnect.metadata.sync.hours", 24) * 3600
+  now = as.numeric(Sys.time())
+
+  for (i in 1:nrow(deploys)) {
+    lastSyncTime <- deploys[i, "lastSyncTime"]
+
+    # for legacy dcf files that don't have sync time saved yet
+    if (is.null(lastSyncTime) || is.na(lastSyncTime))
+      lastSyncTime <- deploys[i, "when"]
+
+    # don't sync if within the configured time window
+    if (as.numeric(lastSyncTime) + syncWindow > now) {
+      next
+    }
+
+    # don't sync non-connect apps
+    if (!isConnectInfo(server = deploys[i, "hostUrl"])) {
+      next
+    }
+
+    account <- rsconnect::accountInfo(deploys[i, "account"],
+                                      server = deploys[i, "server"])
+
+    connect <- clientForAccount(account)
+
+    application <- NULL
+
+    # if the app does not exist, delete the file
+    tryCatch({
+      application <- connect$getApplication(deploys[i, "appId"])
+    }, error = function(c) {
+      message(paste("appId", deploys[i, "appId"], "no longer exists, deleting config file"))
+      file.remove(deploys[i, "deploymentFile"])
+      stop("Removed config file ", deploys[i, "deploymentFile"])
+    })
+
+    record <- deploymentRecord(
+      name = deploys[i, "name"],
+      title = application$title,
+      username = deploys[i, "username"],
+      account = deploys[i, "account"],
+      server = deploys[i, "server"],
+      hostUrl = deploys[i, "hostUrl"],
+      appId = deploys[i, "appId"],
+      bundleId = deploys[i, "bundleId"],
+      url = application$url,
+      when = deploys[i, "when"],
+      lastSyncTime = now,
+      metadata = list(
+        asMultiple = deploys[i, "asMultiple"],
+        asStatic = deploys[i, "asStatic"],
+        vanity_url = application$vanity_url
+      )
+    )
+
+    # update the record and save out a new config file
+    writeDeploymentRecord(record, deploys[i, "deploymentFile"])
+  }
+}
