@@ -439,6 +439,7 @@ test_that("inferQuartoInfo correctly detects info when quarto is provided alone"
   quartoInfo <- inferQuartoInfo(
     appDir = "quarto-doc-none",
     appPrimaryDoc = "quarto-doc-none.qmd",
+    appFiles = bundleFiles("quarto-doc-none"),
     quarto = quarto,
     metadata = list()
   )
@@ -448,6 +449,7 @@ test_that("inferQuartoInfo correctly detects info when quarto is provided alone"
   quartoInfo <- inferQuartoInfo(
     appDir = "quarto-website-r",
     appPrimaryDoc = NULL,
+    appFiles = bundleFiles("quarto-website-r"),
     quarto = quarto,
     metadata = list()
   )
@@ -463,6 +465,7 @@ test_that("inferQuartoInfo extracts info from metadata when it is provided alone
   quartoInfo <- inferQuartoInfo(
     appDir = "quarto-website-r",
     appPrimaryDoc = NULL,
+    appFiles = bundleFiles("quarto-website-r"),
     quarto = NULL,
     metadata = metadata
   )
@@ -478,6 +481,7 @@ test_that("inferQuartoInfo prefers using metadata over running quarto inspect it
   quartoInfo <- inferQuartoInfo(
     appDir = "quarto-website-r",
     appPrimaryDoc = NULL,
+    appFiles = bundleFiles("quarto-website-r"),
     quarto = quarto,
     metadata = metadata
   )
@@ -490,6 +494,7 @@ test_that("inferQuartoInfo returns NULL when content cannot be rendered by Quart
   quartoInfo <- inferQuartoInfo(
     appDir = "shinyapp-simple",
     appPrimaryDoc = NULL,
+    appFiles = bundleFiles("shinyapp-simple"),
     quarto = quarto,
     metadata = list()
   )
@@ -519,6 +524,40 @@ test_that("writeManifest: Quarto document includes quarto in the manifest", {
   expect_equal(manifest$metadata$primary_rmd, "quarto-doc-none.qmd")
 })
 
+test_that("writeManifest: Specifying quarto arg includes quarto in the manifest, even with no appPrimaryDoc specified (.qmd)", {
+  quarto <- quartoPathOrSkip()
+
+  appDir <- "quarto-doc-none"
+  appPrimaryDoc = NULL
+  manifest <- makeManifest(appDir, appPrimaryDoc, quarto = quarto)
+
+  expect_equal(manifest$metadata$appmode, "quarto-static")
+  expect_equal(manifest$quarto$engines, "markdown")
+  expect_equal(manifest$metadata$primary_rmd, "quarto-doc-none.qmd")
+})
+
+test_that("writeManifest: Specifying quarto arg includes quarto in the manifest, even with no appPrimaryDoc specified (.Rmd)", {
+  quarto <- quartoPathOrSkip()
+
+  appDir <- "shiny-rmds"
+  appPrimaryDoc = NULL
+  manifest <- makeManifest(appDir, appPrimaryDoc, quarto = quarto)
+
+  expect_equal(manifest$metadata$appmode, "quarto-shiny")
+  expect_equal(manifest$quarto$engines, "knitr")
+  expect_equal(manifest$metadata$primary_rmd, "non-shiny-rmd.Rmd")
+})
+
+test_that("writeManifest: specifying quarto arg with non-quarto app does not include quarto in the manifest", {
+  quarto <- quartoPathOrSkip()
+
+  appDir <- "shinyapp-singleR"
+  appPrimaryDoc = "single.R"
+  manifest <- makeManifest(appDir, appPrimaryDoc, quarto = quarto)
+
+  expect_null(manifest$quarto)
+})
+
 test_that("writeManifest: Quarto shiny project includes quarto in the manifest", {
   quarto <- quartoPathOrSkip()
 
@@ -563,8 +602,52 @@ test_that("writeManifest: Quarto Python-only website gets correct manifest data"
   expect_null(manifest$packages)
 })
 
+test_that("writeManifest: Deploying a Quarto project without Quarto info in an error", {
+  missingQuartoInfoErrorText <- paste(
+    "Attempting to deploy Quarto content without Quarto metadata.",
+    "Please provide the path to a quarto binary to the 'quarto' argument."
+  )
+
+  appDir <- "quarto-website-r"
+  expect_error(
+    makeManifest(appDir, appPrimaryDoc = NULL, quarto = NULL),
+    missingQuartoInfoErrorText
+  )
+})
+
+test_that("writeManifest: Deploying a Quarto doc without Quarto info in an error", {
+  missingQuartoInfoErrorText <- paste(
+    "Attempting to deploy Quarto content without Quarto metadata.",
+    "Please provide the path to a quarto binary to the 'quarto' argument."
+  )
+
+  appDir <- "quarto-doc-none"
+  appPrimaryDoc <- "quarto-doc-none.qmd"
+  expect_error(
+    makeManifest(appDir, appPrimaryDoc = appPrimaryDoc, quarto = NULL),
+    missingQuartoInfoErrorText
+  )
+})
+
+test_that("writeManifest: Deploying R Markdown content with Quarto gives a Quarto app mode", {
+  quarto <- quartoPathOrSkip()
+
+  manifest <- makeManifest("test-rmds", "simple.Rmd", quarto = quarto)
+
+  expect_equal(manifest$metadata$appmode, "quarto-static")
+  expect_equal(manifest$quarto$engines, "knitr")
+  expect_equal(manifest$metadata$primary_rmd, "simple.Rmd")
+})
+
+test_that("writeManifest: Deploying static content with _quarto.yaml succeeds without quartoInfo", {
+
+  manifest <- makeManifest("static-with-quarto-yaml", NULL, quarto = NULL)
+
+  expect_equal(manifest$metadata$appmode, "static")
+})
+
 test_that("writeManifest: Sets environment.image in the manifest if one is provided", {
-  appDir <- "quarto-proj-r-shiny"
+  appDir <- "shinyapp-simple"
 
   manifest <- makeManifest(appDir, appPrimaryDoc = NULL, image = "rstudio/content-base:latest")
   expect_equal(manifest$environment$image, "rstudio/content-base:latest")
